package core

import (
	"encoding/binary"

	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/token"
	"git.akyoto.dev/cli/q/src/types"
)

// TokenToRegister moves a token into a register.
// It only works with identifiers, numbers and strings.
func (f *Function) TokenToRegister(t token.Token, register cpu.Register) (types.Type, error) {
	switch t.Kind {
	case token.Identifier:
		name := t.Text(f.File.Bytes)
		variable, function := f.Identifier(name)

		if variable != nil {
			f.UseVariable(variable)
			f.SaveRegister(register)
			f.RegisterRegister(asm.MOVE, register, variable.Register)
			return variable.Type, nil
		}

		if function != nil {
			f.SaveRegister(register)
			f.RegisterLabel(asm.MOVE, register, function.UniqueName)
			return types.AnyPointer, nil
		}

		return nil, errors.New(&errors.UnknownIdentifier{Name: name}, f.File, t.Position)

	case token.Number, token.Rune:
		number, err := f.Number(t)

		if err != nil {
			return nil, err
		}

		f.SaveRegister(register)
		f.RegisterNumber(asm.MOVE, register, number)
		return types.Int, nil

	case token.String:
		data := t.Bytes(f.File.Bytes)
		data = String(data)

		slice := make([]byte, len(data)+8+1)
		binary.LittleEndian.PutUint64(slice, uint64(len(data)))
		copy(slice[8:], data)

		label := f.AddBytes(slice)
		f.SaveRegister(register)
		f.RegisterLabel(asm.MOVE, register, label)
		return types.String, nil

	default:
		return nil, errors.New(errors.InvalidExpression, f.File, t.Position)
	}
}
