package types

// Struct is a structure in memory whose regions are addressable with named fields.
type Struct struct {
	Package    string
	UniqueName string
	name       string
	fields     []*Field
	size       int
}

// NewStruct creates a new struct.
func NewStruct(pkg string, name string) *Struct {
	return &Struct{
		Package:    pkg,
		UniqueName: pkg + "." + name,
		name:       name,
	}
}

// AddField adds a new field to the end of the struct.
func (s *Struct) AddField(field *Field) {
	s.fields = append(s.fields, field)
}

// FieldByName returns the field with the given name if it exists.
func (s *Struct) FieldByName(name string) *Field {
	for _, field := range s.fields {
		if field.Name == name {
			return field
		}
	}

	return nil
}

// Name returns the name of the struct.
func (s *Struct) Name() string {
	return s.name
}

// Size returns the total size in bytes.
func (s *Struct) Size() int {
	return s.size
}

// Update updates the offsets and structure size.
func (s *Struct) Update(allTypes map[string]*Struct) {
	s.size = 0

	for _, field := range s.fields {
		field.Type = ByName(field.TypeName, s.Package, allTypes)
		field.Offset = s.size
		s.size += field.Type.Size()
	}
}
